/*
 * Implements the printf BIOS call.
 *
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "printf.h"
#include "../psx/interpreter.h"
#include "../psx/memory.h"
#include "../psx/msg.h"

#include <stdio.h>

void psx_bios_printf(uint32_t* GPR)
{
    uint32_t format_addr = GPR[A0];
    int curr_arg = 0;
    char c;

    while ((c = psx_mem_read_8(format_addr++)))
    {
        if (c == '%')
        {
            c = psx_mem_read_8(format_addr++);
            if (c == '%')
            {
                putchar('%');
            }
            else
            {
                uint32_t arg;
                char format[3] = "% ";

                if (curr_arg < 3)
                    arg = GPR[A1 + curr_arg++];
                else
                {
                    uint32_t addr = GPR[SP] + 0x10 + ((curr_arg++ - 3) << 2);
                    arg = psx_mem_read_32(addr);
                }

                switch (c)
                {
                case 's':
                    while ((c = psx_mem_read_8(arg++)))
                        putchar(c);
                    break;
                case 'd':
                case 'i':
                case 'x':
                case 'u':
                case 'p':
                case 'X':
                case 'O':
                case 'D':
                    format[1] = c;
                    printf(format, arg);
                    break;
                case 'c':
                    putchar((char)(arg & 0xFF));
                    break;
                }
            }
        }
        else
        {
            putchar(c);
        }
    }
}
