/*
 * Operations of bits of an OCaml int32.
 *
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <caml/custom.h>
#include <caml/memory.h>
#include <caml/misc.h>
#include <caml/mlvalues.h>
#include <stdint.h>

/*
 * Bits.get n ub lb returns bits ub .. lb of n, counted as below :
 *
 * 31 30 29 28 27 26 25 24 23 22 ... 8  7  6  5  4  3  2  1  0
 * MSB                                                     LSB
 */
CAMLprim value bits_get(value _n, value _ub, value _lb)
{
    unsigned int n, ub, lb;
    
    /* Unpack */
    n = Int32_val(_n);
    ub = Int_val(_ub);
    lb = Int_val(_lb);

    n >>= lb;
    n &= (1 << (ub - lb + 1)) - 1;

    return Val_int(n);
}

/*
 * This structure is not exported... why ?
 */
extern struct custom_operations caml_int32_ops;

/*
 * Bits.switch_endian transforms a LE number into a BE number and
 * vice-versa.
 */
CAMLprim value bits_switch_endian(value _x)
{
    unsigned int x;
    unsigned int x_be;
    CAMLparam1(_x);
    CAMLlocal1(ret);

    x = Int32_val(_x);
    x_be  = (x & 0xFF000000) >> 24;
    x_be |= (x & 0x00FF0000) >> 8;
    x_be |= (x & 0x0000FF00) << 8;
    x_be |= (x & 0x000000FF) << 24;

    ret = caml_alloc_custom(&caml_int32_ops, 4, 0, 1);
    Int32_val(ret) = x_be;

    CAMLreturn(ret);
}

/*
 * Sign-extend an int16 to an int32.
 */
CAMLprim value bits_int32_of_s16(value s16)
{
    int16_t s16_real;
    CAMLparam1(s16);
    CAMLlocal1(ret);

    s16_real = (int16_t)Int_val(s16);

    ret = caml_alloc_custom(&caml_int32_ops, 4, 0, 1);
    Int32_val(ret) = (int32_t)s16_real;

    CAMLreturn(ret);
}
