(*
 * Types used in the PSX executable format.
 *
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 *)

type segment_descriptor = {
    addr : int32 ;
    size : int32
}

type header = {
    (* Magic string : "PS-X EXE" *)
    magic : string ;

    (* Base of the text segment *)
    text_base : int32 ;

    (* Base of the data segment *)
    data_base : int32 ;

    (* First value of the PC "register" *)
    init_pc : int32 ;

    (* First value of the GP register *)
    init_gp : int32 ;

    (* Text, data, bss and stack address and size *)
    text : segment_descriptor ;
    data : segment_descriptor ;
    bss : segment_descriptor ;
    stack : segment_descriptor
}

type t = {
    header : header ;
    code : int32 array
}

module Make (FS : Filesystem.S) : sig
    val parse : FS.stream -> t
end
