(*
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 *)

(*
 * Instruction takes <= two registers and an immediate value.
 *)
module I : sig
    type args = {
        op : int ;
        rs : R3000_reg.t ;
        rt : R3000_reg.t ;
        imm : int
    }
end

(*
 * Instruction is a unconditional jump.
 *)
module J : sig
    type args = {
        op : int ;
        target : int
    }
end

(*
 * Instruction operates on registers.
 *)
module R : sig
    type args = {
        op : int ;
        rs : R3000_reg.t ;
        rt : R3000_reg.t ;
        rd : R3000_reg.t ;
        shift : int ;
        funct : int
    }
end

(*
 * Instruction is a SYSCALL.
 *)
module S : sig
    type args = {
        op : int ;
        code : int ;
        funct : int
    }
end

(*
 * Instruction branches on some register condition.
 *)
module B : sig
    type args = {
        op : int ;
        rs : R3000_reg.t ;
        typ : int ;
        imm : int
    }
end

(*
 * Include the R3000 instructions from the inline macro file.
 *)
type instr =
#define R3000_INSTR(constr, m) \
|   constr of m.args
#include "instrs.inl"
#undef R3000_INSTR

(*
 * Makes a string out of an instruction.
 *
 * Strings are not \n terminated !
 *)
val to_string : instr -> string

(*
 * Decodes an instruction from its int32 value.
 *)
val decode : int32 -> instr
