(*
 * A common interface for filesystem modules, including :
 * - The OS filesystem
 * - The CD-ROM filesystem
 *
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 *)

type seekreq =
|   Begin
|   Absolute of int
|   Relative of int
|   End

module type S = sig
    type stream

    (* Opens a file and returns a stream to it *)
    val open_ : string -> stream

    (* Closes a stream to a file *)
    val close : stream -> unit

    (*
     * Reads max. n bytes in the file, and returns a couple (buffer, read)
     * with read = number of bytes read (could be less than the maximum
     * in case of EOF, for example).
     *)
    val read : stream -> int -> (string * int)

    (* Seeks into the file *)
    val seek : stream -> seekreq -> unit

    (* Tells where we currently are in the file *)
    val tell : stream -> int

    (* Returns true if we are at the end of the file *)
    val eof : stream -> bool
end
