/*
 * Gate between adresses and precompiled code.
 *
 * Provides a way to get the function to execute for a given address.
 *
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "codegate.h"
#include "globals.h"
#include "interpreter.h"
#include "msg.h"

#include <stdlib.h>

/* Helpers */
static inline int in_text_seg(uint32_t addr)
{
    return (addr >= t_seg_addr) && (addr < (t_seg_addr + t_seg_size));
}

static inline int bios_call(uint32_t addr)
{
    return (addr == 0xA0 || addr == 0xB0 || addr == 0xC0);
}

uint8_t* psx_instr_at(uint32_t addr, uint8_t** jumptable)
{
    /* First case : in the code segment, best case, most frequent */
    if (in_text_seg(addr))
    {
        uint32_t offset = (addr - t_seg_addr) >> 2;
        return jumptable[INSTR_NORMAL + offset];
    }
    /* Second case : BIOS call */
    else if (bios_call(addr))
    {
        return jumptable[INSTR_BIOS];
    }

    /* TODO: Here be programs loaded from the disk, etc. */

    /* Other cases : not yet handled ! */
    else psx_fail("unable to translate %08X to an instruction\n", addr);
}
