/*
 * Defines the interpreter global state and the virtual CPU entry points.
 *
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "address.h"
#include "globals.h"
#include "gpu.h"
#include "hardware.h"
#include "interpreter.h"
#include "memory.h"
#include "msg.h"
#include "spu.h"

#include <getopt.h>
#include <signal.h>
#include <stdint.h>
#include <stdio.h>

static int trace_pc = 0;

static void psx_launch(void)
{
    /* Set the initial SP/GP */
    psx_set_reg(GP, init_gp);
    psx_set_reg(SP, s_seg_addr);

    /* Mainloop */
    while (pc)
    {
        if (trace_pc)
            fprintf(stderr, "[pc] %08X\n", pc);

        psx_exec_one();
    }
}

static void read_cmdline_options(int argc, char** argv)
{
    static struct option opts[] = {
        { "trace-pc",           0, &trace_pc,           1 },
        { "log-unhandled-hw",   0, &log_unhandled_hw,   1 },
        { "count-vsync",        0, &count_vsync,        1 },
        { NULL,                 0, NULL,                0 }
    };

    while (getopt_long_only(argc, argv, "", opts, NULL) != -1)
        ;
}

static void cleanup(void)
{
    psx_spu_unload();
    psx_gpu_unload();
    psx_mem_shutdown();

    if (count_vsync)
    {
        psx_message("Mean vsync wait time : %f\n",
                psx_get_vsync_wait_mean());
    }
}

static void sigint_handler(int dummy)
{
    cleanup();
}

int main(int argc, char** argv)
{
    read_cmdline_options(argc, argv);

    psx_mem_init();
    psx_gpu_load();
    psx_spu_load();

    signal(SIGINT, sigint_handler);

    psx_launch();

    cleanup();

    return 0;
}
