/*
 * CPU/SPU plugin communication.
 *
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../config.h"
#include "spu.h"
#include "msg.h"

#include <dlfcn.h>
#include <stdlib.h>

/* Handle to the shared object */
static void* so;

int (*psx_spu_init)(void);
int (*psx_spu_shutdown)(void);
int (*psx_spu_open)(void);
int (*psx_spu_close)(void);
uint16_t (*psx_spu_read_register)(uint32_t);
void (*psx_spu_write_register)(uint32_t, uint16_t);

void psx_spu_load(void)
{
    so = dlopen(LIBDIR "/libspu.so", RTLD_NOW);
    if (!so)
        psx_fail("No SPU plugin installed on this system !\n");

#define LOAD_(sym, sosym) \
    psx_spu_##sym = dlsym(so, "SPU" #sosym); \
    if (!psx_spu_##sym) \
        psx_fail("Unable to find symbol %s in the SPU plugin !\n", #sosym)

#define LOAD(sym) LOAD_(sym, sym)

    LOAD(init);
    LOAD(open);
    LOAD(close);
    LOAD(shutdown);
    LOAD_(read_register, readRegister);
    LOAD_(write_register, writeRegister);

    psx_spu_init();
    psx_spu_open();
}

void psx_spu_unload(void)
{
    psx_spu_close();
    psx_spu_shutdown();

    dlclose(so);
}
