/*
 * Extensions to the standard OCaml LLVM API.
 *
 * Copyright (c) 2009-2010 Pierre "delroth" Bourdon <delroth@gmail.com>
 *
 * This file is part of Meta-morphosis.
 *
 * Meta-morphosis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Meta-morphosis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Meta-morphosis.  If not, see <http://www.gnu.org/licenses/>.
 */

extern "C" {
#include <caml/alloc.h>
#include <caml/memory.h>
#include <caml/mlvalues.h>

/* Not a public function, but needed though */
extern CAMLprim value caml_ml_output(value chan, value buff,
                                     value start, value len);
}

#include <cstdio>
#include <cstring>
#include <llvm/Bitcode/ReaderWriter.h>
#include <llvm/Support/raw_ostream.h>
#include <llvm-c/Core.h>

class raw_ml_ostream : public llvm::raw_ostream
{
    value channel;
    uint64_t pos;

private:
    virtual void write_impl(const char* ptr, size_t size)
    {
        value str;

        str = caml_alloc_string(size + 1);
        memcpy((void*)str, (const void*)ptr, size);
        ((char*)str)[size] = 0;

        caml_ml_output(channel, str, Val_int(0), Val_int(size));

        pos += size;
    }

    virtual uint64_t current_pos() const { return pos; }
    virtual uint64_t current_pos() { return pos; }

public:
    explicit raw_ml_ostream(value chan) : channel(chan)
    {
        caml_register_global_root(&channel);
    }

    virtual ~raw_ml_ostream()
    {
        caml_remove_global_root(&channel);
    }
};

extern "C" CAMLprim value llvmext_print_module(LLVMModuleRef M, value chan)
{
    CAMLparam1(chan);

    raw_ml_ostream os(chan);
    llvm::unwrap(M)->print(os, NULL);

    CAMLreturn(Val_unit);
}

extern "C" CAMLprim value llvmext_write_bitcode_to_chan(
        LLVMModuleRef M, value chan)
{
    CAMLparam1(chan);

    raw_ml_ostream os(chan);
    llvm::WriteBitcodeToFile(llvm::unwrap(M), os);

    CAMLreturn(Val_unit);
}

extern "C" CAMLprim value llvmext_block_address(LLVMValueRef V,
        LLVMBasicBlockRef BB)
{
    llvm::Value* v;

    v = llvm::BlockAddress::get(
            llvm::unwrap<llvm::Function>(V), llvm::unwrap(BB)
    );
    return reinterpret_cast<value>(llvm::wrap(v));
}

#define Builder_val(b) (*(LLVMBuilderRef*)Data_custom_val(b))

extern "C" CAMLprim value llvmext_build_indirectbr(
        LLVMValueRef address,
        value bb_array,
        LLVMBuilderRef builder)
{
    llvm::IndirectBrInst* br;
    CAMLparam1(bb_array);

    value* arr_ = Op_val(bb_array);
    LLVMBasicBlockRef* arr = reinterpret_cast<LLVMBasicBlockRef*>(arr_);

    br = llvm::unwrap(Builder_val(builder))
            ->CreateIndirectBr(llvm::unwrap(address));
    for (int i = 0; i < Wosize_val(bb_array); ++i)
    {
        br->addDestination(llvm::unwrap(arr[i]));
    }

    CAMLreturn(reinterpret_cast<value>(llvm::wrap(br)));
}
